@@ -1,4 +1,6 @@
 from collections import defaultdict
+from logging import LogRecord
+from typing import List, Union, Tuple, Sequence, Callable, Any, Optional
 import atexit
 import logging
 import warnings
@@ -36,27 +38,42 @@ class LogCapture(logging.Handler):
       ``None`` will be used in its place.
 
       If a callable, it will be called with the :class:`~logging.LogRecord`
-      and the value returned will be used as the row..
+      and the value returned will be used as the row.
 
     :param recursive_check:
 
       If ``True``, log messages will be compared recursively by
       :meth:`LogCapture.check`.
 
-    :param ensure_checks_above: The log level above which checks must be made for logged events.
+    :param ensure_checks_above:
+
+      The log level above which checks must be made for logged events.
+      See :meth:`ensure_checked`.
 
     """
 
+    #: The records captured by this :class:`LogCapture`.
+    records: List[LogRecord]
+    #: The log level above which checks must be made for logged events.
+    ensure_checks_above: Optional[int]
+
     instances = set()
     atexit_setup = False
     installed = False
     default_ensure_checks_above = logging.NOTSET
 
-    def __init__(self, names=None, install=True, level=1, propagate=None,
-                 attributes=('name', 'levelname', 'getMessage'),
-                 recursive_check=False,
-                 ensure_checks_above=None
-                 ):
+    def __init__(
+            self,
+            names: Union[str, Tuple[str]] = None,
+            install: bool = True,
+            level: int = 1,
+            propagate: bool = None,
+            attributes: Union[Sequence[str], Callable[[LogRecord], Any]] = (
+                    'name', 'levelname', 'getMessage'
+            ),
+            recursive_check: bool = False,
+            ensure_checks_above: int = None
+    ):
         logging.Handler.__init__(self)
         if not isinstance(names, tuple):
             names = (names, )
@@ -66,12 +83,11 @@ class LogCapture(logging.Handler):
         self.attributes = attributes
         self.recursive_check = recursive_check
         self.old = defaultdict(dict)
-        #: The log level above which checks must be made for logged events.
         if ensure_checks_above is None:
             self.ensure_checks_above = self.default_ensure_checks_above
         else:
             self.ensure_checks_above = ensure_checks_above
-        self.clear()  # declares self.records: List[LogRecord]
+        self.clear()
         if install:
             self.install()
 
@@ -109,13 +125,12 @@ class LogCapture(logging.Handler):
         for record in self.records:
             record.checked = True
 
-    def ensure_checked(self, level=None):
+    def ensure_checked(self, level: int = None):
         """
         Ensure every entry logged above the specified `level` has been checked.
         Raises an :class:`AssertionError` if this is not the case.
 
         :param level: the logging level, defaults to :attr:`ensure_checks_above`.
-        :type level: Optional[int]
         """
         if level is None:
             level = self.ensure_checks_above
@@ -130,14 +145,16 @@ class LogCapture(logging.Handler):
                     'Not asserted ERROR log(s): %s'
                 ) % (pformat(un_checked)))
 
-    def emit(self, record):
-        # record: logging.LogRecord
+    def emit(self, record: logging.LogRecord):
+        """
+        Record the :class:`~logging.LogRecord`.
+        """
         record.checked = False
         self.records.append(record)
 
     def install(self):
         """
-        Install this :class:`LogHandler` into the Python logging
+        Install this :class:`LogCapture` into the Python logging
         framework for the named loggers.
 
         This will remove any existing handlers for those loggers and
@@ -164,7 +181,7 @@ class LogCapture(logging.Handler):
 
     def uninstall(self):
         """
-        Un-install this :class:`LogHandler` from the Python logging
+        Un-install this :class:`LogCapture` from the Python logging
         framework for the named loggers.
 
         This will re-instate any existing handlers for those loggers
@@ -183,7 +200,7 @@ class LogCapture(logging.Handler):
 
     @classmethod
     def uninstall_all(cls):
-        "This will uninstall all existing :class:`LogHandler` objects."
+        "This will uninstall all existing :class:`LogCapture` objects."
         for i in tuple(cls.instances):
             i.uninstall()
 
@@ -205,7 +222,7 @@ class LogCapture(logging.Handler):
             else:
                 return tuple(values)
 
-    def actual(self):
+    def actual(self) -> List:
         """
         The sequence of actual records logged, having had their attributes
         extracted as specified by the ``attributes`` parameter to the
@@ -214,9 +231,6 @@ class LogCapture(logging.Handler):
         This can be useful for making more complex assertions about logged
         records. The actual records logged can also be inspected by using the
         :attr:`records` attribute.
-
-        :rtype: List
-
         """
         actual = []
         for r in self.records:
@@ -239,15 +253,14 @@ class LogCapture(logging.Handler):
           A sequence of entries of the structure specified by the ``attributes``
           passed to the constructor.
         """
-        result = compare(
+        compare(
             expected,
             actual=self.actual(),
             recursive=self.recursive_check
             )
         self.mark_all_checked()
-        return result
 
-    def check_present(self, *expected, **kw):
+    def check_present(self, *expected, order_matters: bool = True):
         """
         This will check if the captured entries contain all of the expected
         entries provided and raise an :class:`AssertionError` if not.
@@ -265,8 +278,6 @@ class LogCapture(logging.Handler):
           captured entries is required to match those of the expected entries.
           Defaults to ``True``.
         """
-        order_matters = kw.pop('order_matters', True)
-        assert not kw, 'order_matters is the only keyword parameter'
         actual = self.actual()
         expected = SequenceComparison(
             *expected, ordered=order_matters, partial=True, recursive=self.recursive_check
@@ -292,9 +303,9 @@ class LogCaptureForDecorator(LogCapture):
         return self
 
 
-def log_capture(*names, **kw):
+def log_capture(*names: str, **kw):
     """
-    A decorator for making a :class:`LogCapture` installed an
+    A decorator for making a :class:`LogCapture` installed and
     available for the duration of a test function.
 
     :param names: An optional sequence of names specifying the loggers
